@@ -1,6 +1,6 @@
 %%%-------------------------------------------------------------------
 %%% @author Evgeny Khramtsov <ekhramtsov@process-one.net>
-%%% @copyright (C) 2002-2019 ProcessOne, SARL. All Rights Reserved.
+%%% @copyright (C) 2002-2020 ProcessOne, SARL. All Rights Reserved.
 %%%
 %%% Licensed under the Apache License, Version 2.0 (the "License");
 %%% you may not use this file except in compliance with the License.
@@ -361,18 +361,19 @@ url() ->
 url(Schemes) ->
     fun(Val) ->
 	    URL = to_binary(Val),
-	    case http_uri:parse(to_string(URL)) of
-		{ok, {_, _, Host, _, _, _}} when Host == ""; Host == <<"">> ->
+	    case parse_uri(URL) of
+		{ok, _, Host, _} when Host == ""; Host == <<"">> ->
 		    fail({bad_url, empty_host, URL});
-		{ok, {_, _, _, Port, _, _}} when Port =< 0 orelse Port >= 65536 ->
+		{ok, _, _, Port} when Port /= undefined,
+				      Port =< 0 orelse Port >= 65536 ->
 		    fail({bad_url, bad_port, URL});
-		{ok, {Scheme, _, _, _, _, _}} when Schemes /= [] ->
+		{ok, Scheme, _, _} when Schemes /= [] ->
 		    case lists:member(Scheme, Schemes) of
 			true -> URL;
 			false ->
 			    fail({bad_url, {unsupported_scheme, Scheme}, URL})
 		    end;
-		{ok, _} ->
+		{ok, _, _, _} ->
 		    URL;
 		{error, Why} ->
 		    fail({bad_url, Why, URL})
@@ -1143,6 +1144,35 @@ parse_ip_netmask(S) ->
 	    error
     end.
 
+-spec parse_uri(term()) -> {ok, atom(), binary(), integer() | undefined} |
+			   {error, term()}.
+-ifdef(USE_OLD_HTTP_URI).
+parse_uri(URL) when is_binary(URL) ->
+    parse_uri(to_string(URL)); % OTP =< 19's http_uri:parse/1 expects strings.
+parse_uri(URL) ->
+    case http_uri:parse(URL) of
+	{ok, {Scheme, _UserInfo, Host, Port0, Path, _Query}} ->
+	    Port = if is_integer(Port0) -> Port0;
+		      true -> undefined
+		   end,
+	    {ok, Scheme, Host, Port};
+	{error, Reason} ->
+	    {error, Reason}
+    end.
+-else.
+parse_uri(URL0) ->
+    URL = re:replace(URL0, <<"@[A-Z]+@">>, <<"MACRO">>, [{return, binary}]),
+    case uri_string:parse(URL) of
+	URIMap when is_map(URIMap) ->
+	    Scheme = maps:get(scheme, URIMap, <<>>),
+	    Host = maps:get(host, URIMap, <<>>),
+	    Port = maps:get(port, URIMap, undefined),
+	    {ok, to_atom(Scheme), Host, Port};
+	{error, Reason, _Info} ->
+	    {error, Reason}
+    end.
+-endif.
+
 -spec fail(error_reason()) -> no_return().
 fail(Reason) ->
     fail(?MODULE, Reason).
